/*
Cube : The data cube is defined along with its dimensions and measures
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package cube;

import util.*;

public interface Cube extends java.io.Serializable {
    // initialize the cube given db connection information and cube name.
    void init(String connStr, String cubeName) throws Exception; 
    public String name(); // return name of cube.
    public int numDims(); // number of dimensions of the cube.
    public int numLevels(int dimNumber); // number of levels for this dimension. 
    public int numAttrs();  // total number of levels over all dimensions
    public int numMeasures(); // number of measures of the cube.
    public String measure(int i); // name of i-th measure.
    public String attrName(int d, int l); // name of d-th dimension's l-th level 

    // given a dimension and its level, return number of members it has.
    public int numMembers(int dim, int lvl);
    // return i-th member of the given level.
    public Object member(int dim, int lvl, int i);
    // string or external name
    public String memberName(int dim, int lvl, int i);
    // given external name of a member return its internal id.
    public Object memberIdFromName(int dim, int lvl, String extName) throws DataCubeException;
    // Given an internal id, find the corresponding external name
    public String memberNameFromId(int dim, int lvl, Object id) throws Exception;
    // given an attribute id and a parent level, find the parent id.
    public Object getParent(int dim, int lvl, Object attrval, int parLvl) throws Exception;
    
    
    public CubeData subCubeExt(Term t, int axis1, int axis2) throws Exception;
    
    public QueryHandle subCube(Term t, 
			       TermDesc projCols, 
			       MeasureExpr[] measExpr, 
			       int[] sortOrder) throws Exception; 
    public QueryHandle concatCubes(Term t1, Term t2, 
				   TermDesc projCols, 
				   MeasureExpr m1, MeasureExpr m2, 
				   int[] sortOrder)  throws Exception;    
    public QueryHandle concatCubes(Term terms[], 
				   TermDesc projCols, 
				   MeasureExpr measExpr[][], 
				   int[] sortOrder)  throws Exception;
    public QueryHandle subCube(Term t, 
			       TermDesc projCols, 
			       MeasureExpr measExpr[], 
			       QueryHandle subCube, 
			       int[] sortOrder) throws Exception;
    public CubeRows executeQuery(QueryHandle q) throws Exception;
}


