/*
CubeC : Implements the Cube interface
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package cube;

import java.util.*;
import java.io.*;

import util.*;

public abstract class CubeC implements Cube
{
    public int numAttrs() {
	int nattrs = 0;
	for (int i = 0; i < numDims(); i++)
	    nattrs += numLevels(i);
	return nattrs;
    }
    public void output() {
	try {
	    System.out.println("Number dims: " + numDims());
	    for (int i = 0; i < numDims(); i++) {
		System.out.println("Dimension: " + i );
		for ( int j = 0; j < numLevels(i); j++) {
		    System.out.println(attrName(i,j) + " Num members: " + numMembers(i,j));
		    System.out.println("Members of level: " + attrName(i,j));
		    for (int k = 0; k < numMembers(i,j); k++) {
			Object memberId = member(i,j,k);
			String memberName = memberName(i,j,k);
			System.out.println(k + " " + 
					   memberNameFromId(i,j, memberId) + " "
					   + memberIdFromName(i,j,memberName));
		    }
		}
	    }
	    System.in.read();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    
    int position(Vector array, Object val) {
	for (int i = 0; i < array.size(); i++) {
	    if (array.elementAt(i).equals(val))
		return i;
	}
	return -1;
    }
    void insertVal(Vector array, Object val) {
	for (int i = 0; i < array.size(); i++) {
	    int comp = array.elementAt(i).toString().compareTo(val.toString());
	    if (comp == 0)
		return;
	    if (comp > 0) {
		array.insertElementAt(val,i);
		return;
	    }
	}
	array.addElement(val);
    }
    private int findIndex(Vector array, Object obj) throws Exception {
	int p = position(array, obj); 
	if (p >= 0) return p;
	throw new Exception("Element position not found");
    }

    public CubeData subCubeExt(util.Term t, int axis1, int axis2) throws Exception {
	Term t1 = new Term(t);
	t1.validate(this);
	MeasureExpr[] m = new MeasureExpr[1];
	m[0] = new MeasureExpr("sum("+measure(0)+")", "v1"); 
	CubeRows resultSet = executeQuery(subCube(t1,t1.termDescr(),m,null));	
	Vector tuples = new Vector();
	Vector cols = new Vector();
	Vector rows = new Vector();
	Object rowObject = null;
	Object colObject = null;
	while (resultSet.next()) {
	    for (int i = 0; i < t.numAttrs(); i++) {
		Object val = resultSet.getObject(i+1);
		if (i == axis2) {
		    insertVal(cols, val);
		    colObject = val;
		}
		if (i == axis1) {
		    insertVal(rows, val);
		    rowObject = val;
		}
	    }
	    if (rowObject != null)
		tuples.addElement(rowObject);
	    if (colObject != null)
		tuples.addElement(colObject);
	    
	    Object val = resultSet.getObject(numAttrs()+1);
	    tuples.addElement(val);
	}
	int numCols = cols.size();
	int numRows = rows.size();
	AttrDesc rowDesc=null, colDesc=null;
	for (int i = 0; i < t.numAttrs(); i++) {
	    if (i == axis2)
		colDesc = t.desc(i);
	    if (i == axis1)
		rowDesc =  t.desc(i);
	}
	String[] columnNames = new String[numCols];
	String[] rowNames = new String[numRows];
	
	for (int i = 0; i < numCols; i++) {
	    columnNames[i] = memberNameFromId(colDesc.dim(), colDesc.level(), cols.elementAt(i));
	}
	for (int i = 0; i < numRows; i++) {
	    rowNames[i] = memberNameFromId(rowDesc.dim(), rowDesc.level(), rows.elementAt(i));
	}
	
	int c = (numCols == 0)?1:numCols;
	int r = (numRows == 0)?1:numRows;
	if (tuples.size() == 0) {
	    c = 0; r = 0;
	}
	Object[][] data = new Object[r][c]; 
	for (int i = 0 ; i < tuples.size(); i++) {
	    int row = 0;
	    int col = 0;
	    if (numRows > 0)
		row = findIndex(rows, tuples.elementAt(i++));
	    if (numCols > 0)
		col = findIndex(cols, tuples.elementAt(i++));
	    data[row][col] = tuples.elementAt(i);
	}
	resultSet.close();
	return new CubeData(rowNames, columnNames, data);
    }
    public void close() {;}
    public QueryHandle concatCubes(Term t1, Term t2, TermDesc projCols, MeasureExpr m1, MeasureExpr m2, int[] sortOrder) throws Exception {
	Term ts[] = new Term[2];
	ts[0] = t1;
	ts[1] = t2;
	MeasureExpr[][] ma = new MeasureExpr[2][1];
	ma[0][0] = m1;
	ma[1][0] = m2;
	return concatCubes(ts,projCols,ma,sortOrder);
    }
}
