/*
DB : Provides an interface for issuing SQL queries to the database
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package cube;

import util.*;
import java.sql.*;
import java.io.*;

public class DB {
    String _dbname;
    boolean _debug;
    Connection con = null;

    static java.lang.reflect.Method getStmtMethod = null;
    public DB() {
	try {
	    getStmtMethod = Class.forName("java.sql.ResultSet").getMethod("getStatement", null);
	} catch (Exception e) {
	    System.out.println("GetStatement Method is null");
	}
    }

    void printDebug(String st) {
	if (_debug)
	    MyOut.println(st);
    }
    Connection getCon() {return con;} 
    public String dbName() {return _dbname;}
    public String dbType() {return null;}
    public void setAutoCommit(boolean val) {
	try {
	    con.setAutoCommit(val);
	} catch (Exception e) {
	    e.printStackTrace();
	} 
    }
    public void commit() {
	try {
	    con.commit();
	} catch (Exception e) {
	    e.printStackTrace();
	} 
    }
    public void executeUpdate(String st) throws SQLException {
	printDebug(st);
	if (con == null) {
	    MyOut.println("Connection object is null ");
	}
	Statement stmt = con.createStatement();
	stmt.executeUpdate(st);
	stmt.close();
    }
    public void executeUpdate(String st, String asSuffix) throws SQLException {
	executeUpdate(st);
    }
    public ResultSet executeQuery(String st) throws SQLException {
	printDebug(st);
	Statement stmt = con.createStatement();
	return stmt.executeQuery(st);
    }

    public Object executeQuerySingleObject(String st) throws Exception {
	printDebug(st);
	Statement stmt = con.createStatement();
	ResultSet rs = stmt.executeQuery(st);
	if (!rs.next())
	    throw new Exception("DB: Zero result size");
	Object retval = rs.getObject(1);
	rs.close();
	stmt.close();
	return retval;
    }

    public int executeQuerySingleInt(String st) throws Exception {
	printDebug(st);
	Statement stmt = con.createStatement();
	ResultSet rs = stmt.executeQuery(st);
	if (!rs.next())
	    throw new Exception("DB: Zero result size");
	int retval = rs.getInt(1);
	rs.close();
	stmt.close();
	return retval;
    }
    public Object executeQuerySingleInteger(String st) throws Exception {
	printDebug(st);
	Statement stmt = con.createStatement();
	ResultSet rs = stmt.executeQuery(st);
	if (!rs.next())
	    throw new Exception("DB: Zero result size");
	Object retval = rs.getObject(1);
	rs.close();
	stmt.close();
	return retval;
    }
    public PreparedStatement prepareStatement(String st) throws SQLException {
	printDebug(st);
	return con.prepareStatement(st);
    }
    public void dropTable(String relname) {
	String st = "drop table " + relname;
	try {
	    executeUpdate(st);
	} catch (Exception e) {
	    ;
	}
    }
    public void createTempTable(String tabname, String schema) throws Exception {
	executeUpdate("create table " + tabname + schema );
    }
    public void createUnloggedTable(String tabname, String schema) throws Exception  {
	executeUpdate("create table " +  tabname + schema );
    }
    public String nullFilter() {
	return "coalesce";
    }
    public void enableDebug(boolean flag) {
	_debug = flag;
    }
    public static void main(String args[]) {
	try {
	    DB db;
	    if (args[0].equals("DB2")) {
		db = new DB2DB(args[1], args[2], args[3],true);
	    } else {
		db = new OracleDB(args[1], args[2], args[3],true);
	    }
	    if (args.length > 4) {
		MyOut.setOutputFile(args[4]);
	    }
	    DataInputStream din = new DataInputStream(System.in);
	    //	BufferedReader din = new BufferedReader(System.in); 
	    for (int ii = 0; ii < 500; ii++) {
		System.out.println("Next query...");
		String st = din.readLine();
		ResultSet rs = db.executeQuery(st);
		if (rs == null) continue;
		int ncols = rs.getMetaData().getColumnCount();
		while (rs.next()) {
		    String row = "";
		    for (int i = 1; i <= ncols; i++) {
			row += (rs.getString(i) + " ");
		    }
		    MyOut.println(row);
		}
		cursorClose(rs);
	    }
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }
    public static void cursorClose(ResultSet rs) {
	try {
	    if (getStmtMethod == null) {
		rs.close();
		return;
	    }
	    try {
		Statement stmt = (Statement)getStmtMethod.invoke(rs, null);
		rs.close();
		stmt.close();
	    } catch (Exception e) {
		// catch (java.lang.reflect.InvocationTargetException e) {
		getStmtMethod = null;
		System.out.println("Get Statement Method is not supported");
		cursorClose(rs);
	    }

	} catch (Exception e) {

	    e.printStackTrace();
	}
    }
};
