/*
FCubeC : 
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package cube;

import java.util.*;
import java.io.*;

import util.*;

public abstract class FCubeC extends CubeC
{
    String _name;
    int ndims;
    
    class DimInfo implements java.io.Serializable {
	String dimName;
	int nlevels;
	String hierNames[];

	public DimInfo(BufferedReader din)
	{
	    try {
		dimName=din.readLine();
		nlevels=Integer.valueOf(din.readLine()).intValue();
		hierNames=new String[nlevels];
		int i;
		for(i=0;i<nlevels;i++) {
		    hierNames[i] = din.readLine();
		}
	    }
	    catch(Exception e) {
		e.printStackTrace();
	    }
	}
    };
    DimInfo dims[];
    int nmeas;
    String meas[];
    void readParams(BufferedReader din) 
    { 
	try {
	    _name = din.readLine();
	
	    ndims = Integer.valueOf(din.readLine()).intValue();
	    dims = new DimInfo[ndims];
	    for(int i=0; i<ndims; i++)
		dims[i] = new DimInfo(din); 
	
	    nmeas = Integer.valueOf(din.readLine()).intValue();
	    meas = new String[nmeas];
	    for(int i=0; i<nmeas; i++)
		meas[i] = din.readLine();
	}
	catch(Exception e) {
	    e.printStackTrace();
	}
    }
    public void init(String s, String e) throws Exception {;}
    public FCubeC(String ifile)  throws Exception {
	try {
	    BufferedReader din = new BufferedReader(new FileReader(ifile));
	    readParams(din);
	    din.close();
	}
	catch(Exception e) {
	    e.printStackTrace();
	}
    }
    public FCubeC(BufferedReader din)  throws Exception {
	readParams(din);
    }
    public String name() {return _name;}
    public int numDims() {return ndims;}
    public int numLevels(int i) {return dims[i].nlevels;}
    public String attrName(AttrDesc desc) {
	return dims[desc.dim()].hierNames[desc.level()];
    }
    public String attrName(int d, int l) {
	return dims[d].hierNames[l];
    }
    public AttrDesc attrDesc(String attrName) {
	for (int i = 0; i < numDims(); i++) {
	    for (int j = 0; j < numLevels(i); j++) {
		if (dims[i].hierNames[j].equals(attrName)) {
		    AttrDesc adesc =  new AttrDesc();
		    adesc.init(i,j,attrName);
		    return adesc;
		}
	    }
	}
	return null;
    }
    public int dimIndex(String dimName) throws Exception {
	for (int i = 0; i < numDims(); i++) {
	    if (dims[i].dimName.equalsIgnoreCase(dimName))
		return i;
	}
	throw new Exception("Dimension not found");
    }
    public int numMeasures() {return meas.length;}
    public String measure(int i) {return meas[i];}
    class HierStat implements java.io.Serializable {
	Object members[];
	String names[];
	int numMembers() {return members.length;}
	Object member(int pos) {return members[pos];}
	String memberName(int pos) {return names[pos];}
    };
    class DimStat implements java.io.Serializable {
	HierStat hierStat[];
	DimStat(int nlvls) {
	    hierStat = new HierStat[nlvls];
	    for (int i = 0; i < nlvls; i++)
		hierStat[i] = new HierStat();
	}
    };
    DimStat dimStat[];
    // given a dimension and its level, return number of members it has.
    public int numMembers(int dim, int lvl) {return dimStat[dim].hierStat[lvl].numMembers();}
    // return i-th member of the given level.
    public Object member(int dim, int lvl, int i) 
    {return dimStat[dim].hierStat[lvl].member(i);}

    public String memberName(int dim, int lvl, int i) 
    {return dimStat[dim].hierStat[lvl].memberName(i);}    
    public String memberNameFromId(int dim, int lvl, Object id)  throws Exception {
	for (int i = 0; i < numMembers(dim, lvl); i++) {
	    if (member(dim, lvl, i).toString().equals(id.toString()))
		return memberName(dim, lvl, i);
	}
	throw new DataCubeException("Name not found for id: " + id.toString() + " Dim: " + dim + " level: " + lvl);
    }
    void initCubeStat() {
	dimStat = new DimStat[numDims()];
	for (int i = 0; i < numDims(); i++) {
	    dimStat[i] = new DimStat(numLevels(i));
	}
    }
}
