/*
MDCubeC : Handles access to the multidimensional data cube attributes
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package cube;
import java.util.*;
import java.io.*;

import msado15.*;
import msado21.*;
import msadomd.*;
import com.ms.com.*;
import com.ms.wfc.data.*;
import util.*;

public class MDCubeC extends CubeC
{
    String _name;
    int ndims;
   	msado21.Connection cn=null;
	msado21.Connection cn1=null;
	msado21.Connection cn2=null;
	msadomd.Catalog ct=null;
	msadomd.CubeDef cb=null;
	msadomd.Cellset cst=null;
	msado21.Recordset mr=null;
	String ConStr;
	static boolean conUsed;
	
    class DimInfo implements java.io.Serializable {
	
	String dimName;
	int nlevels;
	String hierNames[];

	public DimInfo(int index)
	{
	    Variant var=new Variant(index);
	    try {
			dimName=cb.getDimensions().getItem(var).getName(); 
			nlevels=cb.getDimensions().getItem(var).getHierarchies().getItem(new Variant(0)).getLevels().getCount()-1;
			hierNames=new String[nlevels];
			
			for(int i=0;i<nlevels;i++) {
				hierNames[i] = cb.getDimensions().getItem(var).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(index,i))).getName();   
			}
			
	    }catch(Exception e) {
			e.printStackTrace();
	    }
	}
    };
	
    DimInfo dims[];
    int nmeas;
    String meas[];
    
	void readParams() 
    { 
	try {
	    dims = new DimInfo[ndims];
		for(int i=0; i<ndims; i++) {
			dims[i] = new DimInfo(i);
		}
		
		nmeas=cb.getDimensions().getItem(new Variant("Measures")).getHierarchies().getItem(new Variant(0)).getLevels().getCount();
		meas=new String[nmeas];
		for (int j=0; j<nmeas; j++) {
			meas[j]=cb.getDimensions().getItem(new Variant("Measures")).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(j)).getName();
		}
	}
	catch(Exception e) {
	    e.printStackTrace();
	}
    }
	public MDCubeC(String ConnectionString,String CubeName) throws Exception {
		init(ConnectionString, CubeName);
	}
    public void init(String ConnectionString,String CubeName) throws Exception {
		cn=new msado21.Connection();
		cn.Open(ConnectionString,"","",0);
		
	    ConStr=ConnectionString;
		
		cn1=new msado21.Connection();
		cn2=new msado21.Connection();
		cn1.Open(ConStr,"","",0);
		cn2.Open(ConStr,"","",0);
		
        conUsed=false;
		
		ct=new msadomd.Catalog();
	    ct.setActiveConnection(cn);
		
		cb=ct.getCubeDefs().getItem(new Variant(CubeName)); 
				 
		ndims=cb.getDimensions().getCount();
		
		_name=CubeName; 
		readParams();
    }
    
    public String name() {return _name;}
    public int numDims() {return ndims;}
    public int numLevels(int i) {
		try {
			return dims[i].nlevels;
		}catch (java.lang.NullPointerException ex) {ex.printStackTrace(); }
		return -1;
	}
	public String attrName(int d, int l) {
		try {
			return dims[d].hierNames[l];
		}catch(Exception ex3) {ex3.printStackTrace(); }
		return "";
    }
	
    public int dimIndex(String dimName) throws Exception {
	for (int i = 0; i < numDims(); i++) {
	    if (dims[i].dimName.equalsIgnoreCase(dimName))
		return i;
	}
	throw new Exception("Dimension not found");
    }
    public util.AttrDesc attrDesc(String attrName) {
	try {
	    for (int i = 0; i < numDims(); i++) {
		for (int j = 0; j < numLevels(i); j++) {
		    if (dims[i].hierNames[j].equals(attrName)) {
			util.AttrDesc adesc =  new util.AttrDesc();
			adesc.init(i,j,attrName);
			return adesc;
		    }
		}
	    }
	}catch(java.lang.NullPointerException ex4) {ex4.printStackTrace(); }
	return null;
    }
	public int mslevel(int dimIndex, int lvl) {
	    int totalLevels=cb.getDimensions().getItem(new Variant(dimIndex)).getHierarchies().getItem(new Variant(0)).getLevels().getCount()-1;
		int mslvl=totalLevels-lvl;
		return mslvl;
	}
	
	public int mslevel(util.AttrDesc desc) {
		return mslevel(desc.dim(), desc.level());
	}
	
/*	public Object getInternalId(int dimIndex, int lvl, String MemberName) throws DataCubeException {
		lvl=mslevel(dimIndex,lvl);
		msadomd.Members mbrs=cb.getDimensions().getItem(new Variant(dimIndex)).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(lvl)).getMembers();  
		int nmbrs=mbrs.getCount();		
		for (int i=0; i<nmbrs; i++) {
		    msadomd.Member mem = mbrs.getItem(new Variant(i));
		    if (mem.getName().equals(MemberName)) {
			return mem;
		    }
		}
		return null;
	}*/
	
	public Object getInternalId(int dimIndex, int lvl, String MemberName) throws DataCubeException {
		return MemberName;
	}
    public int numMeasures() {return meas.length;}
    public String measure(int i) {return meas[i];}

   /* public Object getParent(util.AttrDesc desc, Object attrId, int parLvl)  throws Exception {
	int lvl=mslevel(desc);
	int dimIndex=desc.dim();
	msadomd.Member mbr=((msadomd.Member)attrId).getParent() ;
	for (int i=1; i<parLvl; i++) {
	     mbr=mbr.getParent();
	}
	return mbr;	   
    }
	*/
	
	public Object getParent(util.AttrDesc desc, Object attrId, int parLvl)  throws Exception {
		msadomd.Member mbr=((msadomd.Member)getMemberFromId(desc,attrId)).getParent();
		for (int i=desc.level()+1; i<parLvl; i++) {
			 mbr=mbr.getParent();
		}
		return getIdFromMember(mbr);	   
	}
	
   	public int numMembers(int dim,int lvl) {
		   return cb.getDimensions().getItem(new Variant(dim)).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(dim,lvl))).getMembers().getCount();
	}
	
	
    // return i-th member of the given level.
    public Object member(int dim,int lvl,int i) {
	try {
	    return cb.getDimensions().getItem(new Variant(dim)).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(dim,lvl))).getMembers().getItem(new Variant(i));
	} catch (Exception e) {e.printStackTrace(); }
	return null;
    }

    public String memberName(int dim,int lvl,int i) {
	return cb.getDimensions().getItem(new Variant(dim)).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(dim,lvl))).getMembers().getItem(new Variant(i)).getName();
    }
	
    public String getNameFromId(int dim, int lvl, Object id) {
		return id.toString();
	}
	
	Object getIdFromMember(Member mem) {
		return mem.getName();
	}
	Object getMemberFromId(util.AttrDesc desc,Object id) throws Exception {
		String MemberName=id.toString();
		int dimIndex=desc.dim();
		int lvl=desc.level();
		lvl=mslevel(dimIndex,lvl);
		msadomd.Members mbrs=cb.getDimensions().getItem(new Variant(dimIndex)).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(lvl)).getMembers();
		msadomd.Member m = mbrs.getItem(new Variant(MemberName));
		if (m != null)
			return m;
		int nmbrs=mbrs.getCount();
		for (int i=0; i<nmbrs; i++) {
		    msadomd.Member mem = mbrs.getItem(new Variant(i));
		    if (mem.getName().equals(MemberName)) {
			return mem;
		    }
		}
		throw new Exception("Member not found");
	}
	
    public CubeRows subCube(util.Term t,int measureId) throws Exception {	     
		util.TimeKeeper t123=new util.TimeKeeper("SubCube");
		String sourceString="";
		util.TermDesc tdesc;
		String meas=cb.getDimensions().getItem(new Variant("Measures")).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(0)).getMembers().getItem(new Variant(measureId)).getUniqueName();
				
		String colClause="{"+meas+"}"+" ON COLUMNS, ";
		String rowClause="";
		for (int i=0;i<t.size();i++) {
			System.out.print(i+t.val(i).toString()+"\t");
		}
		int oldDimIndex,newDimIndex;
		tdesc=t.termDescr();
		int elmSize=t.size();
				
	    cst=new msadomd.Cellset();
		cst.setActiveConnection(ct.getActiveConnection());
		Variant var1=new Variant();
		Variant var2=new Variant();
		var1.noParam();
		var2.noParam();
		
	// Now depending upon the term, set the source for cst
		
		oldDimIndex=t.desc(0).dim(); // will trace change of dimension within a term
		int numOfDims=1; 
		String whereClause="";
		
		String range="";        
		String constant=cb.getDimensions().getItem(new Variant(tdesc.desc(0).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(0)).getMembers().getItem(new Variant(0)).getUniqueName();
        String dimClause="";
		String all=constant;
		boolean flag=false;
		
		
		for (int i=0; i<elmSize; i++) {
			newDimIndex=t.desc(i).dim();
			if (oldDimIndex != newDimIndex) {
				if (dimClause=="") {
					dimClause = "DESCENDANTS("+constant+","+cb.getDimensions().getItem(new Variant(tdesc.desc(i-1).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(tdesc.desc(i-1)))).getUniqueName()+")";
				}
				if (numOfDims>1) {
//					rowClause = "Crossjoin("+rowClause+","+dimClause+");
					rowClause = rowClause+","+dimClause;
		    	} else {
					rowClause=dimClause;
				}
				numOfDims++;
				constant=cb.getDimensions().getItem(new Variant(tdesc.desc(i).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(0)).getMembers().getItem(new Variant(0)).getUniqueName();
	            range="";
				dimClause="";
				all=constant;
				flag=false;
			}
			
			if(t.constant(i)) {
				constant=((msadomd.Member)getMemberFromId(t.desc(i),t.val(i))).getUniqueName();
			}
			else if (t.range(i)) {
				range=cb.getDimensions().getItem(new Variant(tdesc.desc(i).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(tdesc.desc(i)))).getUniqueName();
			} else {
				if (!flag) {
					if (range != "") {
						dimClause = "DESCENDANTS("+constant+","+range+")";
					} else {
						if ((i==0) || (t.constant(i-1)) || (oldDimIndex!=newDimIndex)) {
							if ( (i==0) || (oldDimIndex!=newDimIndex) ) {
								dimClause="INTERSECT("+cb.getDimensions().getItem(new Variant(tdesc.desc(i).dim())).getUniqueName()+".members,{"+constant+"})";
							} else {
								String consLvl=cb.getDimensions().getItem(new Variant(tdesc.desc(i).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(tdesc.desc(i-1)))).getUniqueName();
								dimClause="INTERSECT(DESCENDANTS("+all+","+consLvl+"),{"+constant+"})";
							}
						} 
					}
				}
				flag=true;
			}
			oldDimIndex=newDimIndex;
		}
		
		if (dimClause == "") {
			dimClause = "DESCENDANTS("+constant+","+cb.getDimensions().getItem(new Variant(tdesc.desc(elmSize-1).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(tdesc.desc(elmSize-1)))).getUniqueName()+")";
		}
		if (numOfDims>1) {
			rowClause = "nonemptycrossjoin("+rowClause+","+dimClause+")";
		} else {
			rowClause=dimClause;
		}
		
		rowClause += " ON ROWS ";
		sourceString="SELECT "+colClause+rowClause+" FROM "+_name;
		System.out.println(sourceString);
		cst.setSource(sourceString);
		cst.Open(var1,var2);
			
// Number of Axis will always be 2 coz of the nature of query
				
		cube.MDCubeRowsC mdc=new cube.MDCubeRowsC(cb,cst,t);
		t123.stop();
		System.out.println(t123.toString());
	   	return mdc;	
	}


	public CubeRows subCube2(util.Term t,int measureId) throws  Exception {
		
	     String sourceString="";
		 String whereClause="";
		 String selectClause="";
		 String measClause=cb.getDimensions().getItem(new Variant("Measures")).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(0)).getMembers().getItem(new Variant(measureId)).getUniqueName();
		 int oldDimIndex=t.desc(0).dim();
		 int newDimIndex;
		 Variant var1=new Variant();
		 Variant var2=new Variant();
		 var1.noParam();
		 var2.noParam();
		 
		 for (int i=0;i<t.size();i++) {
			System.out.print(i+t.val(i).toString()+"\t");		 	 
		 }
		 String all=cb.getDimensions().getItem(new Variant(t.desc(0).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(t.desc(0).dim(),t.desc(0).level()))).getMembers().getItem(new Variant(0)).getUniqueName();
		 String constant=all;
		 String consLvl=cb.getDimensions().getItem(new Variant(t.desc(0).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(t.desc(0).dim(),t.desc(0).level()))).getUniqueName();
		 String aggLvl=consLvl;
		 String range="";
		 String dimClause="";
		 String str="";
		 boolean flag=false;
		 
		 for (int i=0; i <= t.size()-1; i++) {
			 newDimIndex=t.desc(i).dim();
			 
			 if (oldDimIndex != newDimIndex) {
			
				 if (t.range(i-1)) {
					str=consLvl+"='"+constant+"'";		 
					whereClause=(whereClause=="")?"("+str+")":whereClause+" AND ("+str+")";
				 }
				 if (t.constant(i-1)) {
					str=consLvl+"='"+constant+"'";		 
					whereClause=(whereClause=="")?"("+str+")":whereClause+" AND ("+str+")";
					dimClause=consLvl;
				 }
					 
				selectClause=(selectClause=="")?dimClause:dimClause+","+selectClause;
								 
				all=cb.getDimensions().getItem(new Variant(t.desc(i).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(0)).getMembers().getItem(new Variant(0)).getUniqueName();
				constant=all;
				consLvl=cb.getDimensions().getItem(new Variant(t.desc(i).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(0)).getUniqueName();
				aggLvl=consLvl;
				range="";
				dimClause="";
				flag=false;
			 }
			 
			 if (t.constant(i)) {
				 constant=((msadomd.Member)getMemberFromId(t.desc(i),t.val(i))).getUniqueName();
				 consLvl=cb.getDimensions().getItem(new Variant(t.desc(i).dim())).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(t.desc(i).dim(),t.desc(i).level()))).getUniqueName();
			 }
			 if (t.range(i)) {
				 range=cb.getDimensions().getItem(new Variant(newDimIndex)).getHierarchies().getItem(new Variant(0)).getLevels().getItem(new Variant(mslevel(newDimIndex,t.desc(i).level()))).getUniqueName();
				 if (dimClause=="") {
					 dimClause=range;
				 } else {
					dimClause+=","+range;
				 }
			 }
			 if (t.aggregated(i)) {
				 if (!flag) {
					if ((i==0) || (oldDimIndex != newDimIndex)) {
						str=aggLvl+"='"+all+"'";
						whereClause=(whereClause=="")?"("+str+")":whereClause+" AND ("+str+")";
						dimClause=aggLvl;
					} else {
						str=consLvl+"='"+constant+"'";		 
						whereClause=(whereClause=="")?"("+str+")":whereClause+" AND ("+str+")";
						if (t.constant(i-1)) {
							dimClause+=consLvl;
						}
					 	if (t.range(i-1)) {
						    dimClause+=range;
						}
					}
				 }
				 flag=true;
			 }
			oldDimIndex=newDimIndex;
		 }
		 if (t.range(t.size()-1)) {
			 str=consLvl+"='"+constant+"'";		 
			 whereClause=(whereClause=="")?"("+str+")":whereClause+" AND ("+str+")";
		 }
		 if (t.constant(t.size()-1)) {
		     str=consLvl+"='"+constant+"'";		 
		     whereClause=(whereClause=="")?"("+str+")":whereClause+" AND ("+str+")";
		     dimClause+=consLvl;
		 }
		 selectClause = dimClause + ", " + selectClause;
		 selectClause += (","+measClause);
		 sourceString="SELECT "+selectClause+" FROM "+_name+" WHERE ("+whereClause+")";

		 mr=new msado21.Recordset();
		 mr.setSource(sourceString);
		 System.out.println("------------------------------------------------");
		 System.out.println(sourceString);
		 System.out.println("------------------------------------------------");
		 
		 if (!conUsed) {		
			mr.setActiveConnection(cn1);
			conUsed=true;
		 } else {
			mr.setActiveConnection(cn2);
			conUsed = false;
		 }
		
		 mr.Open(var1,var2,AdoEnums.CursorType.STATIC,AdoEnums.LockType.BATCHOPTIMISTIC,AdoEnums.CommandType.TEXT);
		 MDCubeRowsC1 mdc1=new MDCubeRowsC1(cb,mr,t);
		 System.out.println("Returning object");
		 return mdc1;	
		 
    }
	
    public Iter mergedSubCube(Term t1, Term t2, TermDesc tdesc) throws Exception {
//		mergedSubCubeQuery(t1,t2);
		CubeRows iter1 = subCube(t1,0);
		CubeRows iter2 = subCube(t2,0);
		return new MergeDataIter(iter1, iter2, tdesc, t1); 
    }
	
	public Iter mergedSubCubeQuery(Term t1, Term t2) throws Exception {
		int differ=0;
		for (int i=0;i<t1.size();i++) {
			if (t1.val(i)!=t2.val(i)) {
				differ=i;
			}
		}
		
		
		return null;
	}
	
    public void close() {cst.Close();}
}
