/*
ETupleNode : Find the solution that keeps the error within x%.
    Currently just use a simple solution where we find the solution for
    a large value of N and then find the N that meets the error guarantees..
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package diff;

import java.util.*;

import util.*;

// 22 Aug 2000..  Find the solution that keeps the error within x%.
// Currently just use a simple solution where we find the solution for
// a large value of N and then find the N that meets the error guarantees..

    class ETupleNode extends ENode
    {
	double cst0;
	double cst1;
	double prevR;

	ETupleNode(DiffParams pr) {
	    super(pr);
	    _maxErr = pr.maxError();
	}
	public void init(Tuple tup) {
	    _currTup.copy(tup); cst1 = 0; prevR=-2;
	} 
	double absPercentError(double b, double a, double r) {
	    if ((a <= Double.MIN_VALUE) && (b <= Double.MIN_VALUE)) 
		// empty tuple..
		return 0;
	    if ((a <= Double.MIN_VALUE) && (r == Tuple.LARGE_RATIO()))
		// aggregation of missing values.
		return 0;
	    if ((b <= Double.MIN_VALUE) && (r == 0))
		// aggregation of missing values.
		return 0;
	    if ((a <= Double.MIN_VALUE) || (a <= Double.MIN_VALUE)) {
		// laplace correction
		return Math.abs(a*r-b)/(b+1);
	    }
	    return Math.abs(a*r-b)/(b+1);
	}
	int size(double r) {
	    // TODO --- change this to take error function as a parameter..
	    if (absPercentError(_currTup._v2, _currTup._v1, r) <= _maxErr)
		return 0;
	    return 1;
	}
	public double cost(double r) {
	    return (size(r)==0)?error(r):0;
	}
	public double error(double r) {
	    if (prevR == r)
		return cst0;
	    prevR = r;
	    cst0 =  _currTup.error(r);
	    return cst0;
	}	
	public double coveredCost(double r) 
	{return (size(r)==0)?0:cst1;}

	public Tuple aggTup(double r)  { 
	    if (size(r) == 1) return _currTup; else return EmptyTup;
	}
	public Tuple tup(double r, int i)  {
	    return _currTup;
	}
    };


