/*
FullResult : 
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package diff;

import util.*;
import java.util.*;
import cube.*;

public class FullResult extends ResultC implements CoverStatus {
    Vector tups;
    Vector terms; // internal representation of terms..
    double _cost;
    Term t1, t2;
    Tuple retval;
    
    public FullResult(DiffResult res, Term tt1, Term tt2, Cube cube) {
	tups = new Vector(res.numTups());
	terms = new Vector(res.numTups());
	addTuples(res,cube);
	_cost = res.cost();

	t1 = tt1;
	t2 = tt2;
	if (t1 == null)
	    return;
	retval = new Tuple(t1);
	retval.setExternalNames(cube);

	// set the name of the two measure columns;
	String meas1 = null, meas2 = null;
	for (int i = 0; i < t1.numAttrs(); i++)
	    if (!t1.attrVal(i).same(t2.attrVal(i))) {
			System.out.println(t1.val(i).toString() + " " + t2.val(i).toString());
		set1stMeasureName(t1.getExternalName(cube, i));
		set2ndMeasureName(t2.getExternalName(cube, i));
	    }
	/*
	for (int i = 0; i < t1.numAttrs(); i++) {
	    if (t1.getString(i) == null) 
		t1.setString(i, t1.getExternalName(cube,i));
	    retval.setString(i,t1.getString(i));
	    } */
    }
    public FullResult(DiffResult res, Cube cube) {
	this(res,null,null,cube);
    }
    void addTuples(DiffResult res, Cube cube) {
	for (int i = 0; i < res.numTups(); i++) {
	    terms.addElement(new Term(res.tup(i)));
	    Tuple t = new Tuple(res.tup(i));
	    t.setExternalNames(cube); 
	    t.setError(res.tup(i).aggError());
	    tups.addElement(t);
	}
    }
    String _firstName = "Actual";
    String _secondName = "Expected";
    public String firstMeasureName() {return _firstName;}
    public String secondMeasureName() {return _secondName;}
    public  void set1stMeasureName(String name) {if (name != null) _firstName = name;}
    public  void set2ndMeasureName(String name) {if (name != null) _secondName = name;}
    public int numTups() {return tups.size();}
    public ETerm tup(int i) {return (ETerm)tups.elementAt(i);}
    public double cost() {return _cost;}
    public float error(int i) {return (float)((ETerm)tups.elementAt(i)).storedError();}
    public Term firstTerm() {return t1;}
    public Term secondTerm() {return t2;}
    public ETerm fullTuple(int row) {
	if (t1 == null)
	    return tup(row);

	ETerm t = tup(row);
	retval.copyMeasures(t);
	for (int i = 0, j = 0; i < t1.numAttrs(); i++) {
	    if (t1.range(i)) {
		retval.setVal(t.val(j++), i);
	    } else if (t1.constant(i) && !t1.attrVal(i).same(t2.attrVal(i))) {
		retval.setVal(Term.RANGE, i);
	    }
	} 
	return retval;
    }
    public boolean covered(Term t) {
	for (int i = 0; i < numTups(); i++) {
	    if (t.isEqual((Term)terms.elementAt(i), null))
		return true;
	}
	return false;
    }
}
