/*
IENode : Find the solution that keeps the error within x%.
    Currently just use a simple solution where we find the solution for
    a large value of N and then find the N that meets the error guarantees.
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package diff;

import java.util.*;

import util.*;

// 22 Aug 2000..  Find the solution that keeps the error within x%.
// Currently just use a simple solution where we find the solution for
// a large value of N and then find the N that meets the error guarantees..


    class TupleList {
	Vector _tups;
	int sz;
	TupleList() {
	    _tups = new Vector();
	     sz = 0;
	}
	Tuple elementAt(int i) {return (Tuple)_tups.elementAt(i);}
	int size() {return sz;}
	void removeAll() {sz = 0;}
	void insertAt(int i, Tuple t) {
	    Tuple tup;
	    if (_tups.size() > sz) {
		tup = (Tuple)_tups.lastElement();
		tup.copy(t);
		_tups.removeElementAt(_tups.size()-1);
	    } else {
		tup = new Tuple(t);
	    }
	    _tups.insertElementAt(tup,i);
	    //	    System.out.println(elementAt(i).toString());
	    sz++;
	}
	void add(Tuple t) {
	    if (_tups.size() > sz) {
		elementAt(sz).copy(t);
	    } else {
		_tups.addElement(new Tuple(t));
	    }
	    sz++;
	}
	
    }
    
    class Soln implements DiffResult {
	double _cst = 0;
	double _coveredCost;
	TupleList _tups; // array of tuple pointer
	Tuple _aggTup; // agg of all tuples included in the solution list, does not
	Tuple tmp;

	public Soln(double cst){init(cst);}
	public void init(double cst) {
	    _cst = cst; _coveredCost = cst; 
	    _tups = new TupleList();
	    _aggTup = new Tuple();
	    tmp = new Tuple();
	}

	public void start() {_cst = _coveredCost = 0;_aggTup.initMeasures();_tups.removeAll();} 
	// cost not affected by outside ratio..
	public double coveredCost() {return _coveredCost;} 
	public double cost() { return _cst;}
	public int size() {return _tups.size();}
	public Tuple aggTup() {return _aggTup;}
	public int numTups() {return size();}
	public Tuple tup(int i) {
	    return tuple(i);
	}
	public float error(int i) {
	    return (float)tuple(i).error();
	}
	public Tuple tuple(int i) {
	    return _tups.elementAt(i);
	}
	public void updateSoln(double cst, double r,  Soln sol, ENode sol2) {
	    _cst = cst;
	    _coveredCost = sol.coveredCost() + sol2.coveredCost(r);
	    if (sol != this) {
		_aggTup.copyMeasures(sol.aggTup());
		_tups.removeAll();
		for (int i = 0; i < sol.size(); i++)
		    _tups.add(sol.tuple(i));
	    }
	    _aggTup.addChild(sol2.aggTup(r));
	    int sz2 = sol2.size(r);
	    for (int j = 0; j < sz2; j++)
		_tups.add(sol2.tup(r, j));
	}
	public void updateSoln(double cst, Soln sol, Tuple par) {
	    _cst = cst;
	    _coveredCost = cst;
	    if (sol != this)
		_tups.removeAll();
	    _tups.insertAt(0,par);
	    tuple(0).removeChild(sol.aggTup());
	    _aggTup.copyMeasures(par);
	    if (sol == this)
		return;
	    for (int i = 0; i < sol.size(); i++)
		_tups.add(sol.tuple(i));
	}
	public void updateCoveredSoln(double cst, Soln sol, Tuple par) {
	    _cst = cst;
	    _coveredCost = cst;
	    _aggTup.copyMeasures(par);
	    if (sol == this)
		return;
	    _tups.removeAll();
	    for (int i = 0; i < sol.size(); i++)
		_tups.add(sol.tuple(i));
	}

	public  double actualCost(double rp, Soln sol1, Tuple par) {
	    tmp.copyMeasures(par);
	    tmp.removeChild(sol1.aggTup());
	    // if actual R close to the r for which children's cost is
	    // calculated return cached cost..  if (absVal(m.ratio() -
	    // rp) < TOLERANCE) return sol1.cost(); otherwise
	    // recompute cost.
	    return sol1.coveredCost() + tmp.cost();
	}
	public void updateCost(Tuple par, double newR) {
	    _cst = actualCost(par,newR); 
	}
	public double actualCost(Tuple par, double newR) {
	    if (coveredCost() == cost())
		return cost();
	    tmp.copyMeasures(par);
	    tmp.removeChild(aggTup());
	    if (tmp.count() < 0) 
		output();
	    return coveredCost() + tmp.error(newR);
	}

	void output(double finalR) {
	    MyOut.println(" size " + size() + " cost " + cost());
	    for (int i = 0; i < size(); i++) {
		MyOut.println( tuple(i).toString() + " " + ((Tuple)tuple(i)).error(finalR));
	    }
	}
	public void output() {
	    output(_aggTup.ratio());
	}
    };

    class IENode extends ENode implements INode {
	public static final float TOLERANCE = (float)0.1;
	int _maxNum; // current maximum slots <= _N
	Soln _soln[];
	boolean rootNode;
	int minRind; // ratio correspoinding to smallest r for maxNum
	double _prevR;
	int prevInd;
	boolean covered = false;

	RVals _rval;
	public  IENode(DiffParams pr) {
	    super(pr);
	    _soln = null;	    
	    _rval = new RVals();
	    init(pr);
	}
	public void setAsRoot() {rootNode = true;}
	public void init(DiffParams pr){
	    rootNode = false;
	    _rval.init(pr.ratio(), pr.maxR(), pr.ratios());
	    _maxNum = 0;
	    _soln = new Soln[_rval.size()];
	    for (int i = 0; i < _rval.size(); i++) {
		_soln[i] = new Soln(0);
	    }
	}
	public double coveredCost(double r) {return soln(r).coveredCost();}
	public int size(double r) {
	    return soln(r).size();
	}
	public double cost(double r)  //best cost for  external ratio r
	{ 
	    if (_prevR == r) 
		return soln(prevInd).cost();
	    _prevR = r;
	    prevInd = _rval.closestRInd(r);
	    if (Math.abs(r-_rval.val(prevInd))/_rval.val(prevInd) < TOLERANCE)
		return soln(prevInd).cost();
	    return soln(prevInd).actualCost(_currTup,r);
	}
	public double cost(int i) //best cost for  external ratio r
	{    return soln(i).cost();  }
	public Soln soln(double r) {
	    return _soln[_rval.closestRInd(r)];
	}
	public Soln soln(int i) {
	    return _soln[i];
	}
	public Tuple aggTup(double r)  
	{return soln(r).aggTup();}
	public Tuple tup(double r, int i)  
	{return (Tuple)soln(r).tup(i);}
	public double bestRatio() {
	    int minsize = Integer.MAX_VALUE;
	    double minR = Tuple.LARGE_RATIO();
	    for (int i = 0; i < _rval.size();i++) {
		if (soln(i).size() >= minsize)
		    continue;
		minR = _rval.val(i);
		minsize = soln(i).size();
	    }
	    return minR;
	}
	public void setRVals(RVals rvals) {_rval.setRVals(rvals);}
	public void start(Node cnode, BitSet id) // initialize starting costs
	{
	    _maxNum = 0; 
	    _prevR = -1;
	    ((ENode)cnode).setRVals(_rval);
	    _currTup.copy(cnode.tuple());
	    _currTup.mask(id);
	    _currTup.initMeasures(); 
	    // re-init the r-values
	    for (int i = 0; i < _rval.size(); i++) {
		    _soln[i].start();
	    }
	    add(cnode);
	}
	public void add(Node cNode)  // include new node.
	{
	    ENode cnode = (ENode)cNode; 
	    _currTup.addChild(cnode.tuple());
	    // find best solution excluding the parent node.
	    // for the last R, calculate the best value over all r-s
	    int ri = _rval.size()-1;
	    // find best r as the ratio due to the aggTup - explicit tuples
	    // in the merge of these two solutions.	

	    double bestR;
	    // TODO -- computation of bestR faulty --- should use 
	    // the r that leads to smallest total size..
	    if (_currTup.count() == cnode.tuple().count()) {
		// first tuple of this node..
		bestR = cnode.bestRatio();
	    } else {
		 bestR = _rval.getBestR(_currTup, soln(ri).aggTup(),
					cnode.aggTup(_rval.val(ri))); 
	    }
	    double mincost = cost(bestR) + cnode.cost(bestR);
	    soln(ri).updateSoln(mincost, bestR, soln(bestR), cnode);
	    _rval.update(bestR, ri);

	    // Propagate this rval to the child node for better guess on rval
	    // for next tuple..
	    cnode.setRVals(_rval);

	    int rind = 0;
	    for (double r = _rval.val(0); rind < _rval.size()-1; 
		 r = _rval.val(++rind)) {
		soln(rind).updateSoln(soln(rind).cost() + cnode.cost(r), 
				      r, soln(rind), cnode);
	    }
	}
	Soln InFeasibleSoln = new Soln(Double.MAX_VALUE);
	public void close() {
	    // For each r-value and j, update the cost to be the
	    // minimum of the cost with or without the tuple at that
	    // node included.  Find the minimum cost if the root node
	    // at this level is included.
	    int minsize = Integer.MAX_VALUE;
	    double minR = 0;
	    int rind = 0;
	    int minRi = 0;
	    double mincost = Double.MAX_VALUE;
	    for (double rp = _rval.val(0); rp != -1; rp = _rval.val(++rind)) {
		int sz = soln(rind).size()+(covered?0:1);
		if (sz > minsize)
		    continue;
		double cost = InFeasibleSoln.actualCost(rp, soln(rind), _currTup);
		if ((sz == minsize) && (cost > mincost))
		    continue;
		mincost = cost;
		minsize = sz;
		minR = rp;
		minRi = rind;
	    }
	    if (rootNode) {
		minRind = minRi;
	    } 
	    int ri = 0;
	    for (double r = _rval.val(0); r != -1; r = _rval.val(++ri)) {
		if ((soln(ri).size() > minsize) || (rootNode && (ri == minRi))) {
		    if (covered)
			soln(ri).updateCoveredSoln(mincost, soln(minRi), _currTup);
		    else
			soln(ri).updateSoln(mincost, soln(minRi), _currTup);
		}
	    }
	    if (debug) {
		soln(minRi).output(minR);
	    }
	}
	public void setCoveredStatus(boolean status) {
	    covered = status;
	}
	public DiffResult finalResult() {
	    if (debug) {
		MyOut.println( "Min Cost " + soln(minRind).cost());
	    }
	    return soln(minRind);
	}
	public DiffResult getResult(int numSlots) {
	    return soln(minRind);
	}
    };




