/*
<one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 19yy  <name of author>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


package diff;

public class PoissonAlg extends Alg {
    //    public AlgType type()  {return POISSON;}
    public int numDerived()  {return 1;}
    public String derivedCols()  {return ", f1";}
    public String derivedCalcs()  {
	return  ", sum(case when v2 = 0 then 0 else v2*log(v2/" + Alg.msgValSub() + ") end) as f1 ";
    }
    public String filterClause (double ratio)  {
	double descLen = avgDataDescLen();
	double lratio = Math.log(ratio);
	String sratio = String.valueOf(ratio);
	return "(f1 + c1*" + sratio + " - c2*" + String.valueOf(lratio) + ")> 2*numNulls*" + String.valueOf(descLen);
    }
    public void calcDerivedCols(double _v1, double _v2, double _dvals[])
    {
	_dvals[0] = (_v2 < Double.MIN_VALUE)?0:_v2*Math.log(_v2/msgValSub(_v1,_v2));
    }

    public double error(double _v1, double _v2, int cnt, double _dvals[], double r) {
	// find the log data probability assuming a poisson distribution 
	// (e^{-y}*y^k)/k!  with mean y = r*a and k=b. 
	double b =  _v2;
	double a =  _v1;
	double logvals = _dvals[0];
	if (r <= Double.MIN_VALUE) return 0;
	if (a <= Double.MIN_VALUE) {
	    if (b <= Double.MIN_VALUE) // empty tuple..
		return 0;
	    // interchange role of a and b and invert r, implies _dvals = 0, b = 0 	    // a = b; b = msgValSub(_v2, _v1); r = 1/r; logvals = 0;

	    if (r >= LARGE_RATIO) 
		return b;
	    a = msgValSub(_v1, _v2);
	}
	double pp = Math.log(r);
	if (Double.isNaN(r)) {
	    //	    COUT << " WARNING nan(r) " << r << endl;
	}
	double p = logvals - b*pp + r*a;
	// double p = logvals - b*pp;
	//	System.out.println("logvals " + logvals + " r " + r + " logr " + pp + " a " + a + " b " + b);
	if (p >= 0)
	    return p;
	else {
	    if (Double.isNaN(pp)) {
		//		output(p);
		//		COUT << " WARNING -log(prob) < 0 " << p << " " << r << " " << _dvals[0] << endl;
	    }
	}
	return 0;
    }
};




