/*
<one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 19yy  <name of author>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package diff;
import util.*;


public class Tuple extends ETerm {
    // when first value is zero use this value as infty
    static Alg alg = new KLDist();
    double _dvals[] = null;

    public Tuple() {
	super((TermDesc)null);
	initMeasures();
    }
    /*    public Tuple(int numCols) {
	super(numCols);
	_dvals = new double[numDerived()];
	initMeasures();
	} */
    public Tuple(TermDesc tdesc) {
	super(tdesc);
	initMeasures();
    }
    public Tuple(Term t) {
	super(t);
	initMeasures();
    }
    public Tuple(Tuple t) {
	super(t);
	_dvals = new double[numDerived()];
	copyMeasures(t);
    }

    public int numDerived() {return alg.numDerived();}
    public double aggError() {
	if (numNonAgg() == numAttrs()) 
	    return 0;
	return alg.errorRatio(_v1, _v2, _cnt,_dvals,ratio())*100;
    }
    public double error(double r) {return alg.error(_v1,_v2,count(),_dvals,r);}
    public static void setErrorFunction(Alg a) {alg = a;}
    public double error() {return error(ratio());}
    public double cost() {return cost(ratio());}
    public double cost(double r) {
	return error(r) + (double)numNonAgg()*alg.avgDataDescLen();
    }
    public double ratio() {
	return getRatio(_v1,_v2);
    }
    public static double getRatio(double v1, double v2) {
	return alg.getRatio(v1,v2);
    }
    public void calcDerivedCols() {
	alg.calcDerivedCols(_v1, _v2, _dvals);
    }
    public static double LARGE_RATIO() {return alg.LARGE_RATIO;}
    public void initMeasures() {
	super.initMeasures();
	if ((_dvals == null) || (_dvals.length < numDerived()))
	    _dvals = new double[numDerived()];
	for (int i = 0; i < numDerived(); _dvals[i++] = 0);
    }

    public void setMeasures(double v1, double v2) {
	super.setMeasures(v1,v2);
	calcDerivedCols();
    }

    public void copyMeasures(Tuple tup)
    {
	super.copyMeasures(tup);
	if (_cnt < 0)
	    MyOut.println("WARNING cnt <= 0 " + toString());
	if ((_dvals == null) || (_dvals.length < numDerived()))
	    _dvals = new double[numDerived()];
	for (int dv = 0; dv < numDerived(); dv++)
	    _dvals[dv] = tup._dvals[dv];
    }

    public void addChild( Tuple child) {
	_v2 += child._v2; _v1 += child._v1;
	_cnt += child._cnt;
	if (_cnt < 0)
	    MyOut.println("WARNING cnt <= 0");
	for (int i = 0; i < numDerived(); i++)
	    _dvals[i] += child._dvals[i];
    }
    public void removeChild(Tuple child) {
	// assert(notDetailed());
	_v2 -= child._v2; _v1 -= child._v1;
	_cnt -= child._cnt;
	if (_cnt < 0) {
	    Exception e = new Exception("WARNING cnt <= 0 " + toString() + " T " + child.toString());
	    e.printStackTrace();
	}
	for (int i = 0; i < numDerived(); i++)
	    _dvals[i] -= child._dvals[i];
    }
    public void copy(Tuple tup)
    {
	super.copy(tup);
	copyMeasures(tup);
    }
    public String toString() {
	return super.toString() + ", " + ratio()*100 + ", " + error() + ", " + _dvals[0];
    }
};




