/*
ICubeOps : The 3 operators: DIFF, RELAX and INFORM are invoked from here and the results are displayed in a table 
    Copyright (C) 2001 Sunita Sarawagi, Gayatri Sathe

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package frontend;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.colorchooser.*;
import javax.swing.filechooser.*;
import javax.accessibility.*;
import javax.swing.table.*;
import javax.swing.tree.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.io.*;
import java.applet.*;
import java.net.*;

import cube.*;
import main.*;
import util.*;
import relax.*;

class MaxEntFront {
    JProgressBar percentKnownBar;
    Term anchorTerm;
    JButton nextViewButton = null;
    JLabel progressLabel;
    ICubeOps par;
    Term memLvl;

    class InfoResultTableModel extends ResultTableModel {
	InfoResult relRes;
	InfoResultTableModel(InfoResult r) {init(r);}
	void init(InfoResult r) {
	    relRes = r;
	    fireTableChanged(null);
	}
	public Term getTermAt(int row) {
	    return relRes.fullTuple(row);
	}
	public int getColumnCount() { 
	    if ((relRes != null) && (relRes.numTups() > 0))
		return relRes.tup(0).numAttrs() + relRes.numMeasures(); 
	    return 0;
	}
	public int getRowCount() { 
	    if ((relRes != null) && (relRes.numTups() > 0))
		return relRes.numTups();
	    return 0;
	}
	public Object getValueAt(int row, int col) {
	    Term t = relRes.tup(row);
	    if (col < t.numAttrs())
		return t.getString(col);
	    else
		return relRes.measure(row, col-t.numAttrs());
	}
	public String getColumnName(int col) {
	    if (col >= relRes.tup(0).numAttrs()) 
		return relRes.measureName(col-relRes.tup(0).numAttrs());
	    return relRes.tup(0).desc(col).name();
	}
	public Class getColumnClass(int c) {
	    return getValueAt(0, c).getClass();
	}
	public boolean isCellEditable(int row, int col) {return false;}
	public void setValueAt(Object aValue, int row, int column) { return; }
    };
    void showInfoResult(InfoResult res) {
	ResultTableModel m = new InfoResultTableModel(res);
	par.setTableModel(m, m);
    } 
    void addMenus(JMenu menu) {
	JMenuItem mitem = (JMenuItem) menu.add(new JMenuItem("Maxent parameters"));
        mitem.setMnemonic('M');
	mitem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    String pars = JOptionPane.showInputDialog(par.panel, "Semi-colon seperated parameters for Maxent:");
		    try {
			par.server().setMaxentParams(pars);
		    } catch (Exception exp) {
			exp.printStackTrace();
		    }
		}
	    });
    }
    
    public void init(JToolBar toolBar, JToolBar tableToolBar, JPanel ipanel, JMenu menu,ICubeOps cubeOps) {
	par = cubeOps;
	addMenus(menu);
	JButton p = ICubeFront.addTool(toolBar, "Mark seen",true,null);
	p.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    markSeen(par.cubeState().getContext());
		}
	    });
	    
	p = new JButton("Update");
	p.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    getInformationContent();
		}
	    });
	tableToolBar.add(p);

	p = ICubeFront.addTool(tableToolBar, "Mark seen",true,null);
	p.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    markSeenTable();
		}
	    });
	    
	p = ICubeFront.addTool(toolBar, "Top surprises",true,null);
	p.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    try {
			par.putMessage("Getting TopN surprises..");
			Result res = par.server().topNSurprises();
			par.putMessage("Done");
			if (res != null) {
			    res.output();
			} else {
			    System.out.println("Empty result");
			}
			par.showResult(res);
		    } catch (Exception exp) {
			exp.printStackTrace();
		    }
		}
	    });
	p = ICubeFront.addTool(toolBar, "Informative member",true,null);
	p.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    try {
			Term t = new Term(par.cubeState().getContext());
			if (memLvl == null) {
			    memLvl = t;
			    progressLabel.setText("Select drill down level");
			    return;
			}
			InfoResult res = par.server().memberInfoQuery(t, memLvl,true,-1);
			memLvl = null;
			if (res == null) {
			    return;
			}
			showInfoResult(res);
		    } catch (Exception exp) {
			exp.printStackTrace();
		    }
		}
	    });

	p = ICubeFront.addTool(toolBar, "Informative view",true,null);
	p.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    try {
			anchorTerm = new Term(par.cubeState().getContext());
			Term t = par.server().informativeNeibr(anchorTerm);
			if (t != null) {
			    par.cubeState().setContext(t);
			    nextViewButton.setEnabled(true);
			} 
		    } catch (Exception exp) {
			exp.printStackTrace();
		    }
		}
	    });


	p = ICubeFront.addTool(toolBar, "Next view",true,null);
	nextViewButton = p;
	nextViewButton.setEnabled(false);
	p.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    try {
			if (anchorTerm == null)
			    return;
			par.putMessage("Getting next informative view..");
			markSeen(par.cubeState().getContext());
			Term t = par.server().informativeNeibr(anchorTerm);
			par.putMessage("Done");
			if (t != null) {
			    par.cubeState().setContext(t);
			} else {
			    nextViewButton.setEnabled(false);
			}
		    } catch (Exception exp) {
			exp.printStackTrace();
		    }
		}
	    });
	    
	JPanel progressPanel = new JPanel();
	progressPanel.setLayout(new BoxLayout(progressPanel, BoxLayout.Y_AXIS));

	//	    progressPanel.setPreferredSize(new Dimension(400, 20));
	percentKnownBar = new JProgressBar(0,100);
	percentKnownBar.setStringPainted(true);
	percentKnownBar.setString("Information");

	progressPanel.add(percentKnownBar);
	ipanel.add(progressPanel, BorderLayout.NORTH);
    }

    void getInformationContent() {
	try {
	    par.putMessage("Updating information....");
	    float val = par.server().getInformationContentPercent();
	    par.putMessage("Done");
	    setPercentKnown((int)Math.rint(val));
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    void markSeen(Term t) {
	try {
	    par.server().markSeen(t);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    void markSeenTable() {
	Result res = par.getResult();
	if (res == null)
	    return;
	for (int i = 0; i < res.numTups(); i++) {
	    markSeen(res.fullTuple(i));
	}
    }
    void setPercentKnown(int val) {percentKnownBar.setValue(val);}
};
