/*
BoolRelation : Implements Relation for finding Boolean generalizations
    Copyright (C) 2001 Gayatri Sathe 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package relax;

import diff.*;
import util.*;

class BoolRelation implements Relation {
    String relation=new String("boolean");
    boolean isLessThan=false;
    float k = 1;	
    public BoolRelation(boolean less, float _k){
	isLessThan = less;
	k = _k;
    }
    public void display(){
	System.out.println("rel="+relation+" sign="+isLessThan);
    }
    
    public void setSign(boolean less){
	isLessThan = less;
    }
    public void setMeasures(float f1, float f2) {
	isLessThan = (f1 < f2);
    }

    public void setDiffParams(DiffParams diffPars){
	diffPars._alg = new BooleanError();
        diffPars._maxR = 2;
        float temp[] = new float[diffPars._maxR];
        diffPars._ratios = temp;
        if (isLessThan) {
             diffPars._ratios[0] = (float)1.5;
             diffPars._ratios[1] = (float)0.5;
        } else {
             diffPars._ratios[0] = (float)0.5;
             diffPars._ratios[1] = (float)1.5;
        }
	diffPars._exhaustive=false;
    }
 
    public float[] getRS(float[] counts){
	return counts;
    }
			
    public String Rfunc(String v1, String v2){
	if (isLessThan)
	   return "case when (NVL("+v2+","+v1+"+1) >= NVL("+v1+","+v2+"-1)) then 0 else 1 end";
	else 
	   return "case when (NVL("+v1+","+v2+"-1)>=NVL("+v2+","+v1+"+1)) then 0 else 1 end";
    }
	
    public String Sfunc(String v1, String v2){
        return k+"*(1-"+Rfunc(v1,v2)+")";
    }	 
}
