/*
RelaxResult : Stores the final result (generalizations and exceptions) of the relax operator
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package relax;
import java.util.*;
import util.*;
import cube.*;
import diff.*;
public class RelaxResult extends ResultC {
    Vector tups;
    ETerm topT;
    ETerm fullT;
    ETerm firstFullT;
    Term _t1;
    Term _t2;
    BitSetPlus isGen;
    Cube _cube;
    String _firstName = "Actual";
    String _secondName = "Expected";
    
    RelaxResult(MTerm t1, MTerm t2, Vector bitsets, Cube cube) {
	setMeasureName(t1,t2);
	fullT = new ETerm(t1);
	fullT.setExternalNames(cube);
	firstFullT = new ETerm(fullT);
	int size = 0;
	for (int i = 0; i < t1.size(); i++) {
	    if (!t1.attrVal(i).same(t2.attrVal(i))) {
		firstFullT.setVal(Term.RANGE, i);
		continue;
	    }
	    if (t1.aggregated(i))
		continue;
	    size++;
	}
	isGen = new BitSetPlus();
	_t1 = t1;
	_t2 = t2;
	_cube = cube;
	TermDesc tdesc = new TermDesc(size);
	topT = new  ETerm(tdesc);
	size = 0;
	for (int i = 0; i < t1.size(); i++) {
	    if (t1.aggregated(i) || (!t1.attrVal(i).same(t2.attrVal(i))))
		continue;
	    tdesc.desc(size).init(t1.desc(i));
	    topT.setVal(t1.attrVal(i),size);
	    size++;
	}
	topT.setExternalNames(cube);
	topT.setMeasures(t1.measure(), t2.measure());
	if (bitsets == null) {
	    tups = new Vector();
	    return;
	}
	
	tups = new Vector(bitsets.size());
	for (int i = 0; i < bitsets.size(); i++) {
	    addGeneralization((BitSetPlus)bitsets.elementAt(i),(Iter)null);
	}
    }

    
    RelaxResult(ETerm t1, ETerm t2, Vector allTups, BitSetPlus isgenset) {
	_t1 = t1;
	_t2 = t2;
	fullT = new ETerm(t1);
	firstFullT = t1;
	topT = t1;
	tups = allTups;
	isGen = isgenset;
    }

    void setMeasureName(Term t1, Term t2) {
	for (int i = 0; i < t1.numAttrs()+2; i++) {
	    if (!t1.attrVal(i).same(t2.attrVal(i))) {
		_firstName = t1.getString(i);
		_secondName = t2.getString(i);
		break;
	    }
	}
    }
    void addGeneralization(BitSetPlus v, Iter expIter) {
	isGen.set(tups.size());
	// add generalization tuple..

	ETerm temp = new ETerm(topT);
	int pos = 0;
	for (int j = 0; j < fullT.size(); j++) {
	    if (! v.get(j)) {
		continue;
	    }
	    while ((temp.desc(pos).dim() != fullT.desc(j).dim()) || 
		   (temp.desc(pos).level() != fullT.desc(j).level()))
		pos++;
	    temp.setVal(Term.RANGE, pos);
	}
	temp.setMeasures(0,0,v.count());
	tups.addElement(temp);

	if (expIter == null)
	    return;
	// include the generalizations..
	Term genTuple = (Term)tups.elementAt(tups.size()-1);
	while (expIter.moreTuples()) {
	    ETerm t = new ETerm(genTuple);
	    ETerm exp = expIter.getTuple();
	    int expInd = 0;
	    for (int i = 0; i < t.numAttrs(); i++) {
		if (t.range(i)) {
		    t.setVal(exp.val(expInd++), i);
		    t.setString(i, t.getExternalName(_cube,i));
		}
	    }
	    t.copyMeasures(exp);
	    tups.addElement(t);
	}
    }


    void addGeneralization(BitSetPlus v, DiffResult res) {
	isGen.set(tups.size());
	// add generalization tuple..

	ETerm temp = new ETerm(topT);
	int pos = 0;
	for (int j = 0; j < fullT.size(); j++) {
	    if (! v.get(j)) {
		continue;
	    }
	    while ((temp.desc(pos).dim() != fullT.desc(j).dim()) || 
		   (temp.desc(pos).level() != fullT.desc(j).level()))
		pos++;
	    temp.setVal(Term.RANGE, pos);
	}
	temp.setMeasures(0,0,v.count());
	tups.addElement(temp);

	if (res == null)
	    return;
	temp.copyMeasures(res.tup(0));
	// if the diff result produces a generalization of the opposite sign
	// include the generalizations..
	Term genTuple = (Term)tups.elementAt(tups.size()-1);
	for (int j = 1; j < res.numTups(); j++) {
	    ETerm t = new ETerm(genTuple);
	    ETerm exp = res.tup(j);
	    int expInd = 0;
	    for (int i = 0; i < t.numAttrs(); i++) {
		if (t.range(i)) {
		    t.setVal(exp.val(expInd++), i);
		    if (t.constant(i))
		    	t.setString(i, t.getExternalName(_cube,i));
		    else
			t.setVal(Term.RANGE, i);
		}
	    }
	    t.copyMeasures(exp);
	    tups.addElement(t);
	}
    }

    public int numTups() {
	if ((tups != null) && (tups.size() > 0))
	    return tups.size() + 1;
	return 0;
    }
    public ETerm tup(int pos) {
	if (pos == 0)
	    return topT;
	return (ETerm)tups.elementAt(pos-1);
    }
    public ETerm fullTuple(int pos) {
	// return tup(pos);
	if (pos == 0)
	    return firstFullT;
	fullT.copy(firstFullT);
	ETerm temp = tup(pos);
	for (int i = 0,index=0; i < temp.size(); i++) {
	    while ((fullT.desc(index).dim() != temp.desc(i).dim()) ||
		   (fullT.desc(index).level() != temp.desc(i).level()))
		index++;
	    fullT.setVal(temp.val(i),index);
	}
	return fullT;
    }
    public double cost() {return 0;}
    public float error(int i) {return 0;}
    public int count(int row) {
	return tup(row).count();
    }
    public boolean isException(int row) {
	if (row == 0) return false;
	row--;
	return !isGen.get(row);
    }
    public String firstMeasureName() {return _firstName;}
    public String secondMeasureName() {return _secondName;}

}
