/*
RelDataIter : Defines an iterator to get ordered tuples from a subset of the cube 
    Copyright (C) 2001 Sunita Sarawagi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package util;
import java.sql.*;
import cube.*;

public class RelDataIter implements Iter {
    protected CubeRows _rs;
    protected ETerm tup;
    protected int nattrs;
    protected int ntups = 0;
    protected TermDesc tdesc;
    int revMap[];
    public RelDataIter(CubeRows rset, TermDesc td) {
	_rs = rset; 
	tdesc = td;
	nattrs = tdesc.size();
	tup = new ETerm(tdesc);
	revMap = new int[_rs.numColumns()-2];
	
	for (int i = 0; i < revMap.length; revMap[i++] = -1);
	for (int j = 0; j < nattrs; j++) {
	    for (int i = 0; i < revMap.length; i++) {
		if ((tdesc.desc(j).dim() == _rs.colDesc(i).dim()) &&
		    (tdesc.desc(j).level() == _rs.colDesc(i).level())) {
		    revMap[i] = j;
		    break;
		}
	    }
	}
    } 
    public int numAttrs() {return nattrs;}
    public boolean moreTuples() {
	try {
	    if (_rs == null)
		return false;
	    boolean retval = _rs.next();
	    if (retval == false)
		close();
	    return retval;
	}  catch (Exception e) {
	    e.printStackTrace();
	}
	return false;
    }
    public ETerm getTuple() {
	getTuple(tup);
	return tup;
    } 
    public void getTuple(ETerm thisTup) {
	try {
	    for (int i = 0; i < revMap.length; i++) {
		if ((revMap[i] >= 0) && (revMap[i] < nattrs))
		    thisTup.setVal(_rs.getObject(i+1), revMap[i]);
	    }
	    thisTup.setMeasures(_rs.getDouble(_rs.numColumns()-1), _rs.getDouble(_rs.numColumns()));
	}  catch (Exception e) {
	    e.printStackTrace();
	}
	ntups++;
    }
    public TermDesc termDesc() {return tdesc;}
    public void close() {
	try {
	    if (_rs != null)
		_rs.close();
	    _rs = null;
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
    protected void finalize() throws Throwable
    {
	close();
    }
};

